#include <stdio.h>
#include <ros/ros.h>
#include <cstdlib>
#include <visualization_msgs/Marker.h>
#include <geometry_msgs/PoseStamped.h>
#include <geometry_msgs/Quaternion.h>
#include <geometry_msgs/Vector3.h>
#include <math.h>
#include <tf/transform_listener.h>
#include <tf/transform_broadcaster.h>
#include <sensor_msgs/PointCloud2.h>

#define pi 3.14159265

class Ktracker{

public:
	Ktracker();
	ros::Subscriber trkmarker_sub, pose_sub;
	ros::Publisher objectpc_pub;

	void trackCallback(const visualization_msgs::Marker&);
	void poseCallback(const geometry_msgs::PoseStamped&);

	tf::TransformListener listener;
	tf::StampedTransform transform;
	tf::TransformBroadcaster br;
	tf::Quaternion qtf;

private:
	sensor_msgs::PointCloud objectpc;


};

Ktracker::Ktracker(){

	ros::NodeHandle n;
	objectpc_pub=n.advertise<sensor_msgs::PointCloud>("objpc",1000);
}


void Ktracker::trackCallback(const visualization_msgs::Marker &msg){


	objectpc.set_points_size(msg.get_points_size());

	int i;
	for (i=0;i<(int) msg.get_points_size();i++){
		objectpc.points[i].x=msg.points[i].x;
		objectpc.points[i].y=msg.points[i].y;
		objectpc.points[i].z=msg.points[i].z;
	}

	ros::Time now = ros::Time::now();
	transform.setOrigin(tf::Vector3(msg.pose.position.x,msg.pose.position.y,msg.pose.position.z));
	tf::quaternionMsgToTF(msg.pose.orientation,qtf);
	transform.setRotation(qtf);
	br.sendTransform(tf::StampedTransform(transform, msg.header.stamp, "/openni_rgb_frame","/object_frame"));
	objectpc.header.frame_id="/object_frame";

	try
	{
// Enter the correct transformation to track the object 


// 		now=ros::Time::now();
//		listener.waitForTransform("/object_frame", "/end_effector",ros::Time::now(), ros::Duration(0.2));
//		listener.lookupTransform("/object_frame", "/end_effector",now, transform);
//		listener.transformPointCloud("/end_effector",objectpc,objectpc);
		objectpc.header.frame_id="/end_effector";
		objectpc_pub.publish(objectpc);

	}
	catch (tf::TransformException ex)
	{
		ROS_WARN("TF Exception: %s", ex.what());
	}
}


void Ktracker::poseCallback(const geometry_msgs::PoseStamped &msg){
	objectpc_pub.publish(objectpc);
}

int main(int argc, char **argv)
{
	ros::init(argc, argv, "armtrack");
	Ktracker ktracker;
	ros::Rate loop_rate(20);
	visualization_msgs::Marker bluem;
	ros::NodeHandle n;
	ktracker.trkmarker_sub=n.subscribe("markers_out", 1000, &Ktracker::trackCallback,&ktracker);
	ktracker.pose_sub=n.subscribe("armpose",1000, &Ktracker::poseCallback,&ktracker);

	ros::spin();

}
